(function () {

    var _unknownLocation = "_unknown_location";

    return {
        _locationAvailable: true,

        uiCreated: function (namingContainer) {

            var location = dojo.byId(namingContainer + "Location");
            if (location.value == "undefined" || location.value == "null") {
                location.value = "";
            }

            var latitude = dojo.byId(namingContainer + "Latitude");
            if (latitude.value == "undefined" || latitude.value == "null") {
                latitude.value = "";
            }

            var longitude = dojo.byId(namingContainer + "Longitude");
            if (longitude.value == "undefined" || longitude.value == "null") {
                longitude.value = "";
            }

            this.setGeographicCoordinates(namingContainer, {
                location: location.value,
                latitude: latitude.value,
                longitude: longitude.value,
                showLatLng: location.value === _unknownLocation
            });

            var radiusWidget = dijit.byId(namingContainer + "Radius");

            radiusWidget.set("constraints", {min: 0});
            if (!radiusWidget.value && radiusWidget.value !== 0) {
                radiusWidget.set("value", 10);
            }

            var type = this.typeName;
            dojo.require("dijit.form.Button");
            dojo.require("epi.routes");
            var url = epi.routes.getActionPath({
                moduleArea: "CMS",
                controller: "Geographic",
                action: "CoordinatePicker"
            });
            new dijit.form.Button({
                label: dojo.byId(namingContainer + "SelectLocation").value,
                onClick: function () {
                    var lat = dojo.byId(namingContainer + "Latitude").value;
                    var lng = dojo.byId(namingContainer + "Longitude").value;
                    open(url + "?namingContainer=" + escape(namingContainer) + "&lat=" + escape(lat) + "&lng=" + escape(lng) + "&typeName=" + escape(type), "locationPicker", "location=0,status=0,scrollbars=0,width=600,height=500");
                }
            },
            dojo.byId(namingContainer + "SelectLocation"));
        },

        createUI: function (namingContainer, container, settings, containerCallback, criteriaContainerDomNode) {
            function getAvailableCapabilities() {
                return dojo.xhrGet({
                    url: epi.routes.getActionPath({ moduleArea: "CMS", controller: "GeographicLocation", action: "GetAvailableCapabilities" }),
                    handleAs: "json",
                    error: epi.cms.ErrorDialog.showXmlHttpError
                });
            }

            this.languageKeys = "/shell/cms/visitorgroups/criteria/geographiccoordinate/validation/criterianotavailable";
            this.getTranslation(function () {
                getAvailableCapabilities().then(function (regionsVisibility) {
                    function validateCallback() {
                        var initializationErrors = undefined;
                        this._locationAvailable = regionsVisibility.showLocation;
                        if (!regionsVisibility.showLocation) {
                            criteriaContainerDomNode.classList.add("error");
                            initializationErrors = [
                                this.translatedText[
                                    "/shell/cms/visitorgroups/criteria/geographiccoordinate/validation/criterianotavailable"
                                ]
                            ];
                        }
                        containerCallback(namingContainer, container, undefined, initializationErrors);
                    }

                    this.prototype.createUI.call(this, namingContainer, container, settings,
                        validateCallback.bind(this));
                }.bind(this));
            }.bind(this));
        },

        validate: function () {
            var validationErrors = this.prototype.validate.apply(this, arguments);

            if (!this._locationAvailable) {
                validationErrors.Add(this.translatedText["/shell/cms/visitorgroups/criteria/geographiccoordinate/validation/criterianotavailable"], null);
            }
            return validationErrors;
        },

        getSettings: function (namingContainer) {
            return this.prototype.getSettings.apply(this, arguments);
        },

        setGeographicCoordinates: function (namingContainer, values) {

            function roundedNumber(number, decimals) {
                return (Math.round(number * Math.pow(10, decimals)) / Math.pow(10, decimals));
            }

            var location = dojo.byId(namingContainer + "Location");
            location.value = values.location;

            var latitude = dojo.byId(namingContainer + "Latitude");
            latitude.value = values.latitude;

            var longitude = dojo.byId(namingContainer + "Longitude");
            longitude.value = values.longitude;

            if (latitude.value !== "" && longitude.value !== "" && location.value === "") {
                location.value = _unknownLocation;
            }

            var locationPlaceholder = dojo.byId(namingContainer + "LocationPlaceholder");

            locationPlaceholder.style.display = (values.location !== "" && values.location !== _unknownLocation) ? "inline" : "none";

            var latitudeLongitudePlaceholder = dojo.byId(namingContainer + "LatitudeLongitudePlaceholder");

            latitudeLongitudePlaceholder.style.display = values.showLatLng ? "inline" : "none";


            var displayLocation = dojo.byId(namingContainer + "DisplayLocation");
            if (typeof (displayLocation.innerText) == "string") {
                displayLocation.innerText = values.location;
            } else {
                displayLocation.textContent = values.location;
            }

            var latitudeValue = roundedNumber(values.latitude, 3);
            var displayLatitude = dojo.byId(namingContainer + "DisplayLatitude");
            if (typeof (displayLatitude.innerText) == "string") {
                displayLatitude.innerText = latitudeValue;
            } else {
                displayLatitude.textContent = latitudeValue;
            }

            var longitudeValue = roundedNumber(values.longitude, 3);
            var displayLongitude = dojo.byId(namingContainer + "DisplayLongitude");
            if (typeof (displayLongitude.innerText) == "string") {
                displayLongitude.innerText = longitudeValue;
            } else {
                displayLongitude.textContent = longitudeValue;
            }
        }
    };
})();
